// 
// lat - GtkMainWindow.cs
// Author: Loren Bandiera
// Copyright 2005 MMG Security, Inc.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; Version 2 .
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//

using System;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using Gtk;
using Novell.Directory.Ldap;
using Novell.Directory.Ldap.Rfc2251;
using Novell.Directory.Ldap.Utilclass;

#if ENABLE_AVAHI
using Avahi;
#endif

namespace lat
{
	public class GtkMainWindow : Gtk.Window
	{
		protected Gnome.Program program;
		
		protected Gtk.Widget currentWidget;
		protected Gtk.MenuToolButton newMenuToolButton;
		protected Gtk.ToolButton newToolButton;
		protected Gtk.ToolButton propertiesToolButton;
		protected Gtk.ToolButton deleteToolButton;
		protected Gtk.ToolButton refreshToolButton;
		protected Gtk.ToolButton templateToolButton;
		protected Gtk.AccelGroup newAccelGroup;
		
		protected bool showAllAttributes;
		protected string editDN;
		protected string editName;
		protected bool editIsCopy;
		protected TreeIter editIter;
		protected uint sbContextID;

#if ENABLE_AVAHI
		protected ServiceFinder finder;
#endif		
		
		protected Gtk.HPaned hpaned1;
		protected lat.GtkLdapBrowser ldapBrowser;
		protected lat.GtkLdapSchemaBrowser ldapSchemaBrowser;
		protected lat.GtkLdapViews ldapViewsView;
		protected lat.GtkLdapSearch ldapSearch;
		protected Gtk.MenuBar menubar1;
		protected Gtk.Notebook notebook1;
		protected Gtk.Image secureImage;
		protected Gtk.Statusbar statusbar1;
		protected Gtk.Toolbar toolbar1;

		
		public GtkMainWindow(string[] args) : base("")
		{
			program = new Gnome.Program (Defines.PACKAGE, Defines.VERSION, Gnome.Modules.UI, args);
			
			Stetic.Gui.Build(this, typeof(lat.GtkMainWindow));

			// FIXME: doesn't work if I set these in the designer for some reason			
			ldapViewsView.ViewSelected += OnViewSelected;
			ldapBrowser.EntrySelected += OnBrowserEntrySelected;
			ldapSchemaBrowser.EntrySelected += OnSchemaEntrySelected;
			ldapSearch.Export += OnSearchResultExported;
			ldapSearch.SearchResultSelected += OnSearchResultSelected;
			ldapSearch.StatusMessageSent += OnStatusMessageSent;
			
			// show something
			Gtk.Label blankLabel = new Gtk.Label ("");
			blankLabel.Show();
			hpaned1.Pack2 (blankLabel, true, true);
			currentWidget = blankLabel;
			
			// Restore window positions
			LoadPreference (Preferences.MAIN_WINDOW_WIDTH);
			LoadPreference (Preferences.MAIN_WINDOW_X);
			LoadPreference (Preferences.MAIN_WINDOW_MAXIMIZED);
			LoadPreference (Preferences.MAIN_WINDOW_HPANED);
			LoadPreference (Preferences.BROWSER_SELECTION);
			
			// Watch for any changes
			Preferences.SettingChanged += OnPreferencesChanged;

			// Setup toolbar
			GetToolbarButtons ();

			// FIXME: need to be able to create MenuToolButton in designer
			newToolButton.Hide ();
			templateToolButton.Hide ();

			newMenuToolButton = new Gtk.MenuToolButton ("gtk-new");
			newMenuToolButton.Show ();
			toolbar1.Insert (newMenuToolButton, 0);			
			
			ToggleButtons (false);	

			newAccelGroup = new AccelGroup ();
			this.AddAccelGroup (newAccelGroup);

			// Setup status bar
			sbContextID = statusbar1.GetContextId (Defines.PACKAGE);
			
#if ENABLE_NETWORKMANAGER
			Global.Network = NetworkDetect.Instance;
			Global.Network.StateChanged += OnNetworkStateChanged;
#endif

#if ENABLE_AVAHI
			// FIXME: causes delay/crashes on exit for some reason
			finder = new ServiceFinder ();
			finder.Found += new FoundServiceEventHandler (OnServerFound);
			finder.Removed += new RemovedServiceEventHandler (OnServerRemoved);
			finder.Start ();
#endif			
		}
		
		public void Run ()
		{
			program.Run ();
		}
		
		#region internal_methods

		void ChangeCurrentWidget (string widgetType)
		{
			Assembly asm = Assembly.GetExecutingAssembly ();
			Type t = asm.GetType (widgetType);
		
			if (!(currentWidget.GetType() == t)) {
				currentWidget.Destroy ();
				currentWidget = null;

				object obj = Activator.CreateInstance (t);
				currentWidget = (Gtk.Widget) obj;
				currentWidget.Show ();

				hpaned1.Pack2 (currentWidget, true, true);
			}		
		}

		void Close ()
		{
			int x, y, width, height;
			this.GetPosition (out x, out y);
			this.GetSize (out width, out height);

			bool maximized = ((this.GdkWindow.State & Gdk.WindowState.Maximized) > 0);
			Preferences.Set (Preferences.MAIN_WINDOW_MAXIMIZED, maximized);

			if (!maximized) {
				Preferences.Set (Preferences.MAIN_WINDOW_X, x);
				Preferences.Set (Preferences.MAIN_WINDOW_Y, y);
				Preferences.Set (Preferences.MAIN_WINDOW_WIDTH, width);
				Preferences.Set (Preferences.MAIN_WINDOW_HEIGHT, height);
			}

			Preferences.Set (Preferences.MAIN_WINDOW_HPANED, hpaned1.Position);
			Preferences.Set (Preferences.BROWSER_SELECTION, ldapBrowser.BrowserSelectionMethod);

#if ENABLE_AVAHI
			finder.Stop ();
#endif

			program.Quit ();
		}

		Connection GetActiveConnection ()
		{
			string serverName = null;
			Connection conn = null;

			switch (notebook1.CurrentPage) {
			
			case 0:
				serverName = ldapViewsView.GetActiveServerName ();
				break;
				
			case 1:
				serverName = ldapBrowser.GetActiveServerName ();
				break;
				
			case 2:
				serverName = ldapSearch.SelectedServer;
				break;
				
			case 3:
				serverName = ldapSchemaBrowser.GetActiveServerName ();			
				break;
			
			default:
				break;
			}
							
			if (serverName == null)
				return conn;

			conn = Global.Connections [serverName];			
			return conn;
		}

		void GenerateNewMenu (Connection conn)
		{		
			Gtk.Menu newMenu = new Gtk.Menu ();	
 				 	
			foreach (ViewPlugin vp in Global.Plugins.ServerViewPlugins)		
					if (conn.ServerViews.Contains (vp.GetType().ToString())) {
						ImageMenuItem menuitem = new ImageMenuItem (vp.MenuLabel, newAccelGroup);
						menuitem.AddAccelerator ("activate", newAccelGroup, vp.MenuKey);
						
						Gtk.Label l = (Gtk.Label) menuitem.Child;
						l.Text = vp.MenuLabel;
						
						menuitem.Image = new Gtk.Image (vp.Icon);
						menuitem.Activated += OnNewMenuItemActivated;
						menuitem.Show ();						
						newMenu.Append (menuitem);
					} 

//			newMenuItem.Submenu = newMenu;		
			newMenuToolButton.Menu = newMenu;
		}

		void GetToolbarButtons ()
		{
			newToolButton = (ToolButton) toolbar1.GetNthItem (0);
			propertiesToolButton = (ToolButton) toolbar1.GetNthItem (1);
			deleteToolButton = (ToolButton) toolbar1.GetNthItem (2);
			refreshToolButton = (ToolButton) toolbar1.GetNthItem (3);
			templateToolButton = (ToolButton) toolbar1.GetNthItem (4);			
		}

		void LoadPreference (String key)
		{
			object val = Preferences.Get (key);

			if (val == null) {

				if (key == Preferences.MAIN_WINDOW_HPANED)
					hpaned1.Position = 250;

				return;
			}
			
			Log.Debug ("Setting {0} to {1}", key, val);

			switch (key) {
			case Preferences.MAIN_WINDOW_MAXIMIZED:
				if ((bool) val)
					this.Maximize ();
				else
					this.Unmaximize ();
				break;

			case Preferences.MAIN_WINDOW_X:
			case Preferences.MAIN_WINDOW_Y:
				this.Move((int) Preferences.Get(Preferences.MAIN_WINDOW_X),
						(int) Preferences.Get(Preferences.MAIN_WINDOW_Y));
				break;
			
			case Preferences.MAIN_WINDOW_WIDTH:
			case Preferences.MAIN_WINDOW_HEIGHT:
				this.SetDefaultSize((int) Preferences.Get(Preferences.MAIN_WINDOW_WIDTH),
						(int) Preferences.Get(Preferences.MAIN_WINDOW_HEIGHT));

				this.ReshowWithInitialSize();
				break;

			case Preferences.MAIN_WINDOW_HPANED:
				hpaned1.Position = (int) Preferences.Get (Preferences.MAIN_WINDOW_HPANED);
				break;
				
			case Preferences.BROWSER_SELECTION:
				ldapBrowser.BrowserSelectionMethod = (int) val; 
				break;				
			}
		}		
		
		void RemoveToolbarHandlers ()
		{
			if (currentWidget is GtkLdapDataView) {
				newMenuToolButton.Menu = null;
				GtkLdapDataView dv = (GtkLdapDataView) currentWidget;
		
				newMenuToolButton.Clicked -= dv.OnNewEntryActivate;
				propertiesToolButton.Clicked -= dv.OnEditActivate;
				deleteToolButton.Clicked -= dv.OnDeleteActivate;
				refreshToolButton.Clicked -= dv.OnRefreshActivate;
			}
		}
		
		void SetupToolbar (ViewPlugin vp)
		{
			Tooltips t = new Tooltips ();
			string tipMsg = null;

			GtkLdapDataView dv = (GtkLdapDataView) currentWidget;

			tipMsg = String.Format ("Create a new {0}", vp.Name.ToLower());
			newMenuToolButton.SetTooltip (t,  tipMsg, tipMsg);			
			newMenuToolButton.Clicked += dv.OnNewEntryActivate;
			
			tipMsg = String.Format ("Edit the properties of a {0}", vp.Name.ToLower());
			propertiesToolButton.SetTooltip (t,  tipMsg, tipMsg);			
			propertiesToolButton.Clicked += dv.OnEditActivate;
			
			tipMsg = String.Format ("Delete a {0} from the directory", vp.Name.ToLower());
			deleteToolButton.SetTooltip (t,  tipMsg, tipMsg);			
			deleteToolButton.Clicked += dv.OnDeleteActivate;

			tipMsg = "Refreshes the data from the server";
			refreshToolButton.SetTooltip (t,  tipMsg, tipMsg);			
			refreshToolButton.Clicked += dv.OnRefreshActivate;

			ToggleButtons (true);
		}
		
		void ToggleButtons (bool btnState)
		{
			newMenuToolButton.Sensitive = btnState;
			propertiesToolButton.Sensitive = btnState;
			deleteToolButton.Sensitive = btnState;
			refreshToolButton.Sensitive = btnState;

			propertiesToolButton.Show ();
			refreshToolButton.Show ();
		}
		
		void UpdateStatusBar ()
		{
			string msg = null;

			Connection conn = GetActiveConnection ();
			if (conn == null)
				return;

			try {
			
				if (conn.AuthDN == null)
					msg = String.Format("Bind DN: anonymous");
				else
					msg = String.Format("Bind DN: {0}", conn.AuthDN);

				statusbar1.Pop (sbContextID);				
				statusbar1.Push (sbContextID, msg);

				secureImage.Pixbuf = Util.GetSSLIcon (conn.UseSSL);
				
			} catch (Exception e) {
				Log.Debug (e);
			}
		}
		
		void WriteStatusMessage (string msg)
		{
			statusbar1.Pop (sbContextID);
			statusbar1.Push (sbContextID, msg);
		}
		
		#endregion

		protected virtual void OnAboutActivated(object sender, System.EventArgs e)
		{
			Gtk.AboutDialog ab = new Gtk.AboutDialog ();
			ab.Authors = Defines.AB_AUTHOR;
			ab.Comments = Defines.AB_DESCRIPTION;
			ab.Copyright = Defines.AB_COPYRIGHT;
			ab.Documenters = Defines.AB_DOCS;
			ab.Name = Defines.PACKAGE;
			ab.TranslatorCredits = Defines.AB_TRANSLATORS;
			ab.Version = Defines.VERSION;
			ab.Icon = Gdk.Pixbuf.LoadFromResource ("lat.png");

			ab.Run ();
			ab.Destroy ();
		}

		protected virtual void OnBrowserEntrySelected(object o, lat.LdapEntrySelectedEventArgs args)
		{
			Connection conn = Global.Connections [args.Server];
			
			if (args.IsHost) {
			
				ChangeCurrentWidget ("lat.GtkLdapServerInfo");
				
				GtkLdapServerInfo serverInfo = (GtkLdapServerInfo) currentWidget;
				serverInfo.CurrentConnection = conn;
								
				return;
			}

			ChangeCurrentWidget ("lat.GtkLdapAttributeEditor");
	
			GtkLdapAttributeEditor attributeEditor = (GtkLdapAttributeEditor) currentWidget;
			LdapEntry entry = conn.Data.GetEntry (args.DN);			
			if (entry != null)
				attributeEditor.Show (conn, entry, showAllAttributes);
		}

		protected virtual void OnConnectActivated(object sender, System.EventArgs e)
		{
			ConnectDialog cd = new ConnectDialog ();
			cd.Run ();
			cd.Destroy ();
			
			ldapBrowser.Refresh ();
			ldapSchemaBrowser.Refresh ();
			ldapViewsView.Refresh ();
		}

		protected virtual void OnCopyActivated(object sender, System.EventArgs e)
		{
			if (!(notebook1.CurrentPage == 1))
				return;

			Connection conn = GetActiveConnection ();
			if (conn == null)
				return;

			editDN = ldapBrowser.SelectedEntry;
			editName = conn.Settings.Name;
			editIsCopy = true;

			Log.Debug ("Edit->Copy dn: {0}", editDN);
		}

		protected virtual void OnCutActivated(object sender, System.EventArgs e)
		{
			if (!(notebook1.CurrentPage == 1))
				return;

			Connection conn = GetActiveConnection ();
			if (conn == null)
				return;

			editDN = ldapBrowser.SelectedEntry;
			editIter = ldapBrowser.GetSelectedIter ();
			editName = conn.Settings.Name;

			Log.Debug ("Edit->Cut dn: {0}", editDN);
		}

		protected virtual void OnDeleteActivated(object sender, System.EventArgs e)
		{
			if (notebook1.CurrentPage == 0) {
				ChangeCurrentWidget ("lat.GtkLdapDataView");
				GtkLdapDataView dv = (GtkLdapDataView) currentWidget;
				dv.OnDeleteActivate (sender, e);
			 } else if (notebook1.CurrentPage == 1) {
			 	ldapBrowser.OnDeleteActivate (sender, e);
			}
		}

		protected virtual void OnDisconnectActivated(object sender, System.EventArgs e)
		{
			Connection conn = GetActiveConnection ();
			if (conn == null)
				return;
				
			conn.Disconnect ();		
		}

		protected virtual void OnExportActivated(object sender, System.EventArgs e)
		{
			Connection conn = GetActiveConnection ();
			if (conn == null)
				return;
		
			SelectContainerDialog scd = new SelectContainerDialog (conn);
			scd.Title = Mono.Unix.Catalog.GetString ("Export entry");
			scd.Message = Mono.Unix.Catalog.GetString ("Select the container you wish to export.");
			scd.Run ();

			if (scd.DN.Equals (""))
				return;

			Util.ExportData (conn, this, scd.DN);
			scd.Destroy ();
		}

		protected virtual void OnHelpContentsActivated(object sender, System.EventArgs e)
		{
			try {

				Gnome.Help.DisplayDesktopOnScreen (program, 
					Defines.PACKAGE, 
					"lat.xml", 
					null, 
					Gdk.Screen.Default);

			} catch (Exception ex) {

				Log.Debug (ex);
				Util.ShowErrorDialog ("Help error", ex.Message);
			}		
		}

		protected virtual void OnImportActivated(object sender, System.EventArgs e)
		{
			Connection conn = GetActiveConnection ();
			if (conn == null)
				return;
		
			FileChooserDialog fcd = new FileChooserDialog (
				Mono.Unix.Catalog.GetString ("Choose an LDIF file to import"),
				Gtk.Stock.Open, 
				this, 
				FileChooserAction.Open);

			fcd.AddButton (Gtk.Stock.Cancel, ResponseType.Cancel);
			fcd.AddButton (Gtk.Stock.Open, ResponseType.Ok);

			fcd.SelectMultiple = false;

			ResponseType response = (ResponseType) fcd.Run();
			if (response == ResponseType.Ok) {

				UriBuilder ub = new UriBuilder ();
				ub.Scheme = "file";
				ub.Path = fcd.Filename;

				Util.ImportData (conn, this, ub.Uri);
			} 
		
			fcd.Destroy();			
		}

		protected virtual void OnMassEditActivated(object sender, System.EventArgs e)
		{
			MassEditDialog dlg = new MassEditDialog();
			dlg.Run ();
			dlg.Destroy ();
		}

#if ENABLE_NETWORKMANAGER
		void OnNetworkStateChanged (object o, NetworkStateChangedArgs args)
		{
			if (args.Connected) {
			
				this.Sensitive = true;
			
			} else {

				HIGMessageDialog dialog = new HIGMessageDialog (
					this,
					0,
					Gtk.MessageType.Info,
					Gtk.ButtonsType.Ok,
					"Network disconnected",
					"The network is down. LAT will disable itself until it comes back up");

				dialog.Run ();
				dialog.Destroy ();
				
				this.Sensitive = false;				
			}
		}
#endif

		protected virtual void OnNewActivated(object sender, System.EventArgs e)
		{
			if (notebook1.CurrentPage == 0) {
				ChangeCurrentWidget ("lat.GtkLdapDataView");
				GtkLdapDataView dv = (GtkLdapDataView) currentWidget;
				dv.OnNewEntryActivate (sender, e);
			 } else if (notebook1.CurrentPage == 1) {
			 	ldapBrowser.OnNewEntryActivate (sender, e);
			}		
		}

		protected virtual void OnNewMenuItemActivated (object o, EventArgs args)
		{
			ImageMenuItem mi = (ImageMenuItem) o;
			Gtk.Label l = (Gtk.Label) mi.Child;

			GtkLdapDataView dv = (GtkLdapDataView) currentWidget;			
			dv.ShowNewItemDialog (l.Text);
		}

		protected virtual void OnPasteActivated(object sender, System.EventArgs e)
		{
			if (!(notebook1.CurrentPage == 1))
				return;

			Connection conn = GetActiveConnection ();
			if (conn == null)
				return;

			if (conn.Settings.Name != editName) {			
				string msg = Mono.Unix.Catalog.GetString ("Cannot copy/cut enteries between servers");
				Util.ShowErrorDialog ("Paste error", msg);
				return;
			}

			string pasteDN = ldapBrowser.SelectedEntry;
			if (pasteDN == null)
				return;

			DN dn = new DN (editDN);
			RDN r = (RDN) dn.RDNs [0];

			try {

				if (editIsCopy) {
					conn.Data.Copy (editDN, r.toString (false), pasteDN);
					WriteStatusMessage ("Entry copied");
				} else {
					conn.Data.Move (editDN, r.toString (false), pasteDN);
					ldapBrowser.RemoveRow (editIter);
					WriteStatusMessage ("Entry moved.");
				}
				
			} catch (Exception ex) {

				Log.Debug (ex);				
				StringBuilder msg = new StringBuilder ();

				if (editIsCopy)
					msg.AppendFormat ("{0} {1}", Mono.Unix.Catalog.GetString ("Unable to copy entry "), editDN);
				else
					msg.AppendFormat ("{0} {1}", Mono.Unix.Catalog.GetString ("Unable to move entry "), editDN);
				
				msg.AppendFormat ("\nError: {0}", ex.Message);

				Util.ShowErrorDialog ("Paste error", msg.ToString());
			}

			if (editIsCopy)
				editIsCopy = false;
				
			editDN = null;
			editName = null;
		}
		
		protected virtual void OnPreferencesActivated(object sender, System.EventArgs e)
		{
			new PreferencesDialog (program);
			Global.Connections.Save ();
		}
		
		protected void OnPreferencesChanged (object sender, GConf.NotifyEventArgs args)
		{
			LoadPreference (args.Key);
		}

		protected virtual void OnPopulateSambaActivated(object sender, System.EventArgs e)
		{
			PopulateSambaDialog dlg = new PopulateSambaDialog();
			dlg.Run ();
			dlg.Destroy ();
		}

		protected virtual void OnPropertiesActivated(object sender, System.EventArgs e)
		{
			if (notebook1.CurrentPage == 0) {
				ChangeCurrentWidget ("lat.GtkLdapDataView");
				GtkLdapDataView dv = (GtkLdapDataView) currentWidget;
				dv.OnEditActivate (sender, e);
			 }
		}

		protected virtual void OnQuitActivated(object sender, System.EventArgs e)
		{
			Close ();
		}

		protected virtual void OnRefreshActivated(object sender, System.EventArgs e)
		{
			if (notebook1.CurrentPage == 0) {
				ChangeCurrentWidget ("lat.GtkLdapDataView");
				GtkLdapDataView dv = (GtkLdapDataView) currentWidget;
				dv.OnRefreshActivate (sender, e);
			}
		}

		protected virtual void OnReloginActivated(object sender, System.EventArgs e)
		{
			Connection conn = GetActiveConnection ();
			if (conn == null)
				return;		
		
			string msg = Mono.Unix.Catalog.GetString ("Enter the new username and password you wish to re-login with");
			LoginDialog ld = new LoginDialog (conn, msg);
			ld.Run ();

			UpdateStatusBar ();		
		}

		protected virtual void OnSchemaEntrySelected (object sender, LdapSchemaSelectedEventArgs args)
		{
			Log.Debug ("User selected schema: {0}", args.Name);
			
			if (args.Name == "Object Classes" || args.Name == "Attribute Types" || args.Name == "Matching Rules" || args.Name == "LDAP Syntaxes")
				return;
				
			Connection conn = Global.Connections [args.Server];
			
			if (args.Parent == "Object Classes") {
				
				SchemaParser sp = conn.Data.GetObjectClassSchema (args.Name);
				ChangeCurrentWidget ("lat.GtkLdapObjectClassSchema");
				
				GtkLdapObjectClassSchema sw = (GtkLdapObjectClassSchema) currentWidget;
				sw.Clear ();
				sw.Show (sp);

			} else if (args.Parent == "Attribute Types") {
			
				// FIXME: create widget for attribute types
				Util.ShowInfoDialog ("FIXME", "re-create widget for attribute types schema");
				
				//SchemaParser sp = conn.Data.GetAttributeTypeSchema (args.Name);
				//ChangeCurrentWidget ("lat.GtkLdapAttributeTypeSchema");
				
				//GtkLdapAttributeTypeSchema sw = (GtkLdapAttributeTypeSchema) currentWidget;
				//sw.Clear ();
				//sw.Show (sp);
				
			} else if (args.Parent == "Matching Rules") {

				SchemaParser sp = conn.Data.GetMatchingRule (args.Name);
				ChangeCurrentWidget ("lat.GtkLdapMatchingRuleSchema");
				
				GtkLdapMatchingRuleSchema sw = (GtkLdapMatchingRuleSchema) currentWidget;
				sw.Clear ();
				sw.Show (sp);

			} else if (args.Parent == "LDAP Syntaxes") {

				SchemaParser sp = conn.Data.GetLdapSyntax (args.Name);
				ChangeCurrentWidget ("lat.GtkLdapSyntaxSchema");
				
				GtkLdapSyntaxSchema sw = (GtkLdapSyntaxSchema) currentWidget;
				sw.Clear ();
				sw.Show (sp);
			}
		}

		void OnSearchResultExported (object o, SearchResultExportEventArgs args)
		{
			Connection conn = Global.Connections [ldapSearch.SelectedServer];
			if (conn == null)
				return;
				
			if (args.IsDND) {
				string data = null;
				Util.ExportData (conn, args.DN, out data);
				args.Data = data;
			} else { 
				Util.ExportData (conn, this, args.DN);
			}	
		}

		void OnSearchResultSelected (object o, SearchResultSelectedEventArgs args)
		{
			Connection conn = Global.Connections [ldapSearch.SelectedServer];			
			LdapEntry entry = conn.Data.GetEntry (args.DN);

			if (entry != null) {
				ChangeCurrentWidget ("lat.GtkLdapAttributeEditor");
				GtkLdapAttributeEditor attributeEditor = (GtkLdapAttributeEditor) currentWidget;			
				attributeEditor.Show (conn, entry, showAllAttributes);				
			}
		}

#if ENABLE_AVAHI
		void OnServerFound (object o, FoundServiceEventArgs args)
		{
			Global.Connections [args.FoundConnection.Settings.Name] = args.FoundConnection;
			ldapViewsView.AddConnection (args.FoundConnection);
			ldapBrowser.AddConnection (args.FoundConnection.Settings.Name);
			ldapSchemaBrowser.AddConnection (args.FoundConnection.Settings.Name);
		}
		
		void OnServerRemoved (object o, RemovedServiceEventArgs args)
		{
			Global.Connections.Delete (args.ConnectionName);
			ldapViewsView.Refresh ();
			ldapBrowser.Refresh ();
			ldapSchemaBrowser.Refresh ();			
		}
#endif

		protected virtual void OnShowAllAttributesToggled(object sender, System.EventArgs e)
		{
			Connection conn = null;
			showAllAttributes = !showAllAttributes;

			if (notebook1.CurrentPage == 1) {

				ChangeCurrentWidget ("lat.GtkLdapAttributeEditor");
				GtkLdapAttributeEditor attributeEditor = (GtkLdapAttributeEditor) currentWidget;
				
				if (ldapBrowser.SelectedEntry == null || ldapBrowser.SelectedServer == null)
					return;

				conn = Global.Connections [ldapBrowser.SelectedServer];
				LdapEntry entry = conn.Data.GetEntry (ldapBrowser.SelectedEntry);
				attributeEditor.Show (conn, entry, showAllAttributes);
			}		
		}

		protected virtual void OnStatusMessageSent (object sender, StatusMessageSentEventArgs args)
		{
			try {

				statusbar1.Pop (sbContextID);				
				statusbar1.Push (sbContextID, args.StatusMessage);
				
				string connectionName = null;
				
				if (sender is GtkLdapSearch) 
					connectionName = ldapSearch.SelectedServer;
				else if (sender is GtkLdapDataView)
					connectionName = ((GtkLdapDataView) sender).CurrentConnection.Settings.Name;
				
				if (connectionName != null) {
					Connection conn = Global.Connections [connectionName];
					secureImage.Pixbuf = Util.GetSSLIcon (conn.UseSSL);
				}
				
			} catch (Exception ex) {
				Log.Debug (ex);
			}
		}

		protected virtual void OnTemplatesActivated(object sender, System.EventArgs e)
		{
			Connection conn = GetActiveConnection ();
			if (conn == null)
				return;
		
			TemplatesDialog dlg = new TemplatesDialog (conn);
			dlg.Run ();
			dlg.Destroy ();
		}

		protected virtual void OnWindowDelete(object o, Gtk.DeleteEventArgs args)
		{
			Close ();
			args.RetVal = true;
		}

		protected virtual void OnViewSelected(object o, lat.ViewSelectedEventArgs args)
		{
			Log.Debug ("User selected view: {0}", args.Name);
		
			ViewPlugin vp = Global.Plugins.GetViewPlugin (args.Name, args.ConnectionName);
			Connection conn = Global.Connections [args.ConnectionName];
			
			if (vp == null) {
			
				ChangeCurrentWidget ("lat.GtkLdapServerInfo");
				
				GtkLdapServerInfo serverInfo = (GtkLdapServerInfo) currentWidget;
				serverInfo.CurrentConnection = conn;
				UpdateStatusBar ();
				
				return;
			}
			
			ChangeCurrentWidget ("lat.GtkLdapDataView");
			
			GtkLdapDataView dataView = (GtkLdapDataView) currentWidget;
			dataView.StatusMessageSent += OnStatusMessageSent;
			dataView.CurrentConnection = conn;
			dataView.ConfigureView (vp);
			dataView.Populate ();
			
			SetupToolbar (vp);
			GenerateNewMenu (conn);
		}

		protected virtual void OnViewSwitchPage(object o, Gtk.SwitchPageArgs args)
		{
			switch (args.PageNum) {
			
			// views
			case 0:
				ldapBrowser.RemoveToolbarHandlers ();
				ChangeCurrentWidget ("lat.GtkLdapDataView");
				ToggleButtons (false);
				templateToolButton.Hide ();
				break;
				
			// browser
			case 1:				
				RemoveToolbarHandlers ();
				ChangeCurrentWidget ("lat.GtkLdapAttributeEditor");
				ToggleButtons (true);
				templateToolButton.Show ();
				propertiesToolButton.Hide ();
				refreshToolButton.Hide ();
				ldapBrowser.SetToolbarHandlers (newMenuToolButton, deleteToolButton);
				break;
			
			// search
			case 2:
				ldapBrowser.RemoveToolbarHandlers ();
				RemoveToolbarHandlers ();
				ChangeCurrentWidget ("lat.GtkLdapAttributeEditor");
				ToggleButtons (false);
				templateToolButton.Hide ();
				break;
			
			// schema
			case 3:
				RemoveToolbarHandlers ();
				ToggleButtons (false);
				templateToolButton.Hide ();
				break;
			
			default:
				break;
			}
		}		


		
	}

	public delegate void StatusMessageSentHandler (object sender, StatusMessageSentEventArgs args);
	
	public class StatusMessageSentEventArgs : EventArgs
	{
		public string StatusMessage;

		public StatusMessageSentEventArgs (string statusMessage)
		{
			this.StatusMessage = statusMessage;
		}
	}
}
